# import inspect
import json
import logging
import os
import socket
import sys

from datetime import datetime

from singleton_decorator import singleton
from runtime_reporting_utils import constants
from typing import Optional


from runtime_reporting_utils.file_utils import (
    calculate_md5,
    get_file_size,
    is_binary_file,
    get_line_count,
)

# from functools import wraps

# def singleton(cls):
#     instances = {}

#     @wraps(cls)
#     def get_instance(*args, **kwargs):
#         if cls not in instances:
#             # Get the caller's filename (instantiating script)
#             caller_frame = inspect.stack()[1]  # The frame of the caller
#             caller_filename = os.path.abspath(caller_frame.filename)

#             # Create an instance and store the caller's file path
#             instance = cls(*args, **kwargs)
#             instance.instantiator_path = caller_filename  # Store in instance
#             instances[cls] = instance
#         return instances[cls]

#     return get_instance

# def get_caller_path() -> str:
#     """Get the absolute path of the caller.

#     Returns:
#         str: The absolute path of the caller.
#     """
#     frame = inspect.stack()[1]
#     module = inspect.getmodule(frame[0])
#     return os.path.abspath(module.__file__)


@singleton
class Reporter:
    """Class for purpose"""

    def __init__(self, **kwargs):
        """Constructor for class Reporter"""
        self.config = kwargs.get("config", None)
        self.config_file = kwargs.get("config_file", None)
        self.logfile = kwargs.get("logfile", None)

        self.start_time = datetime.now()
        self.username = os.getenv("USER") or os.getenv("USERNAME") or "unknown-user"

        # frame = inspect.currentframe()
        # caller_frame = frame.f_back  # The frame of the caller (instantiator)
        # caller_filename = inspect.getfile(caller_frame)
        self.executable = os.path.abspath(sys.argv[0])

        # self.executable = get_caller_path()
        # print(f"Executable: {self.executable}")
        # import sys
        # sys.exit(1)
        self.outdir = kwargs.get("outdir", None)
        if self.outdir is None:
            self.outdir = os.path.join(
                constants.DEFAULT_OUTDIR_BASE,
                os.path.basename(self.executable),
                constants.DEFAULT_TIMESTAMP,
            )

            logging.info(f"Setting outdir to {self.outdir}")

        if not os.path.exists(self.outdir):
            os.makedirs(self.outdir)

        if self.logfile is not None:
            self.add_file(file=self.logfile, name="log file")

        if self.config_file is not None:
            self.add_file(file=self.config_file, name="config file")

        self._counters = {}
        self._files = {}

        self.hostname = socket.gethostname()
        logging.info(f"Instantiated Reporter in {os.path.abspath(__file__)}")

    # def __del__(self):
    #     """Destructor for class Reporter"""
    #     self.generate_report()

    def add_file(self, file: str, name: str) -> None:
        """Add a file to the report.

        Args:
            path (str): The file to add to the report.
            name (str): The name of the file in the report.
        """
        path = os.path.abspath(file)
        if path not in self._files:
            self._files[path] = {"path": path, "name": name}
        else:
            print(f"This file '{path}' has already been added to the runtime reporter.")

    def count(self, name: str, desc: Optional[str] = None) -> int:
        """Increment the count for the named attribute.

        Args:
            name (str): The name of the attribute being counted.
            desc (Optional[str]): The description to count.

        Returns:
            int: The current count for the name attribute.
        """
        if name not in self._counters:
            self._counters[name] = {"count": 0, "desc": desc}
        self._counters[name]["count"] += 1

    def generate_report(self) -> None:

        for path in self._files:
            # path = self._files[name]['path']
            self._files[path]["bytesize"] = get_file_size(path)
            self._files[path]["is_binary"] = is_binary_file(path)
            if not self._files[path]["is_binary"]:
                self._files[path]["line_count"] = get_line_count(path)
            self._files[path]["md5sum"] = calculate_md5(path)

        lookup = {
            "provenance": {
                "executable": self.executable,
                "hostname": self.hostname,
                "outdir": self.outdir,
                "username": self.username,
                "start_time": self.start_time.isoformat(),
                "end_time": datetime.now().isoformat(),
                "elapsed_time": str(datetime.now() - self.start_time),
            },
            "command-line arguments": sys.argv[1:],
            "counters": self._counters,
            "files": self._files,
            "environment": dict(os.environ),
        }

        outfile = os.path.join(self.outdir, "report.json")

        with open(outfile, "w") as f:
            f.write(json.dumps(lookup, indent=4))

        logging.info(f"Runtime report written to {outfile}")
        print(f"Runtime report written to {outfile}")
